#pragma once
#include "PathMap.h"

class CPathMapWin:
    public CPathMap
{
public:
    CPathMapWin(){}
    ~CPathMapWin(){}

    // Load a bitmap, where white = free, black=impassable, blue=target, any other color - red = danger / 256
    HRESULT LoadBmp( LPCTSTR strFileName, bool loadDangerInfo );

    // Same as above, but loadDangerInfo=false, and loading from resource not from file
    HRESULT LoadBmp( UINT iResID );

    struct CELL_INFO
    {
        bool isPassable;
        u16  cost;
        u8   directionForward;
        bool isDestination;
        bool isOnTheWave;
        bool isDirty;
    }
    getCellInfo( CPoint c )
    {
        const SMapEntry &me = m_map[c];
        CELL_INFO res;
        res.isPassable = ( me.cost != u16Max );
        res.cost = me.cost;
        res.directionForward = me.directionForward;
        res.isDestination = me.isDestination();
        res.isOnTheWave = me.isHot();
        res.isDirty = !me.isDone();
        return res;
    }

    inline CSize GetSize(){ return m_map.GetSize(); }


    inline CPoint GetIndex( size_t ind ) const
    {
        return m_map.GetIndex( ind );
    }

    inline size_t GetIndex( CPoint pt ) const
    {
        return m_map.GetIndex( pt );
    }

    void UpdateWholeGraph();

    struct TurretInfo
    {
        // Position of the upper-right corner
        CPoint Position;
        // Damage, in arbitrary units
        BYTE Damage;
        // Radius in cells
        BYTE Radius;
    };

    void AddTurret( const TurretInfo &t );

    void RemoveTurret( size_t indTurret );

    bool isTurretBody( CPoint pt, size_t* pTurrentIdx ) const;

    inline void CleanupTurrets()
    {
        m_Turrets.SetCount(0);
    }

    // Update the map by taking the values from m_mapMasterCopy, copy to the m_map, and apply all the turrets
    void UpdateMapFromMaster();

    CString GetCellInfo( const CPoint& pt ) const;

    void updateStart();
    int RecalcIncremental( int iMaxPixels, CRect& rcUpdatedArea );
private:
	// Array of turrets the user placed
    CAtlArray<TurretInfo> m_Turrets;

    enum E_BASE_MAP_ENTRY
    {
        CELL_IMPASABLE,
        CELL_PASABLE,
        CELL_DESTINATION
    };
    CArray2D<E_BASE_MAP_ENTRY> m_mapMasterCopy;

	// How many turrets are in the specified cell.
    CArray2D<int> m_TurretsCount;

    typedef void(CPathMapWin::*cellFromColorProc)( const CPoint &cell, RGBTRIPLE c );
    HRESULT LoadBmpImpl( const CArray2D<RGBTRIPLE> &bm, cellFromColorProc proc );

    // Two types of loading
    static const BYTE s_BlackColorThreshold = 16;
    void cellFromColor_Direct( const CPoint &cell, RGBTRIPLE c );
    void cellFromColor_ToMaster( const CPoint &cell, RGBTRIPLE c );

    inline long Sqr( long x ) { return x * x; }

    typedef void(CPathMapWin::*turretUpdateProc)( const CPoint &cell, bool isBody, BYTE damage );
    void processTurret( const TurretInfo& ti, turretUpdateProc proc );
    void turret_Add( const CPoint &cell, bool isBody, BYTE damage );
    void turret_Remove( const CPoint &cell, bool isBody, BYTE damage );
    void turret_UpdateCoverage( const CPoint &cell, bool isBody, BYTE damage );
};