#pragma once

template<class T>
class CArray2D
{
public:
	CArray2D(){}
	~CArray2D(){ m_arr.SetCount(0); }

	bool SetSize(const CSize& szNew)
	{
		m_size = szNew;
		if( !m_arr.SetCount( m_size.cx * m_size.cy ) )
			return false;
		return true;
	}
    inline const CSize &GetSize() const
    {
        return m_size;
    }

	// Execute ZeroMemory for the entire array
	// Don't execute with non-trivial data types (classes with vtable, classes with pointers, etc..)
	inline void ZeroMem() { ZeroMemory( m_arr.GetData(), sizeof(T) * m_arr.GetCount() ); }

	// Execute MemSet for the entire array
	// Don't execute with non-trivial data types (classes with vtable, classes with pointers, etc..)
	inline void MemSet(int val) { memset( m_arr.GetData(), val, sizeof(T) * m_arr.GetCount() ); }

	// Elements accessors
	// Linear indices works a bit faster
	inline T& operator[] ( size_t i ) { return m_arr.GetAt( i ); }
	inline const T& operator[](size_t i) const{ return m_arr.GetAt(i); }

	inline T& operator[]( const CPoint& p ) { return m_arr.GetAt( GetIndex( p ) ); }
	inline const T& operator[]( const CPoint& p ) const { return m_arr.GetAt( GetIndex( p ) ); }

	// Direct data access
	inline const T* GetData() const { return m_arr.GetData(); }
	inline T* GetData() { return m_arr.GetData(); }

	// Conversations between 2D indices and linear indices in CAtlArray
	inline size_t GetIndex( const CPoint& p ) const
	{
		return GetIndex( p.x, p.y );
	}

	inline size_t GetIndex( ptrdiff_t x, ptrdiff_t y ) const
	{
		// debug code
		if( x < 0 || y < 0 || x >= m_size.cx || y >= m_size.cy )
			__debugbreak();
		return x + y * m_size.cx;
	}

	inline CPoint GetIndex( size_t i ) const
	{
		CPoint res;
		res.x = LONG( i % m_size.cx );
		res.y = LONG( i / m_size.cx );
		return res;
	}

	inline size_t GetCount() const
	{
		return m_arr.GetCount();
	}

	inline bool IsOutOfBounds( const CPoint& p ) const
	{
		return p.x < 0 || p.y < 0 || p.x >= m_size.cx || p.y >= m_size.cy;
	}
private:
	CAtlArray<T> m_arr;
	CSize m_size;
};