#include "stdafx.h"
#include "BitmapLoader.h"

HRESULT LoadBitmapFile( CArray2D<RGBTRIPLE> &res, LPCTSTR strFileName )
{
    HBITMAP hBmp = (HBITMAP)LoadImage( 0, strFileName, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_LOADFROMFILE );
    if( NULL == hBmp )
        return GetLastHr();

    BITMAP bm; 
    if( sizeof(bm) != GetObject( hBmp, sizeof(bm), &bm ) )
        return GetLastHr();

     // Must be 24 bit RGB 
    if( 1 != bm.bmPlanes || 24 != bm.bmBitsPixel )
    {
        MyTrace( "The image must be 24 bit\n" );
        return HRESULT_FROM_WIN32( ERROR_INVALID_PIXEL_FORMAT );
    }

    const CSize sz( bm.bmWidth, bm.bmHeight );
    if( sz.cx < 3 || sz.cy < 3 )
    {
        MyTrace( "The image size must be at least 2x2 pixels\n" );
        return HRESULT_FROM_WIN32( ERROR_INCORRECT_SIZE );
    }

    if( !res.SetSize( sz ) )
        return E_OUTOFMEMORY;
    res.ZeroMem();

    const BYTE* pData = (const BYTE*)(bm.bmBits);

    CPoint p;
    for( p.y=0; p.y<sz.cy; p.y++ )
    {
        const size_t szOffset = bm.bmWidthBytes * p.y;
        const RGBTRIPLE* pRgb = (const RGBTRIPLE*)( pData + szOffset );
        for( p.x=0; p.x<sz.cx; p.x++ )
        {
            const RGBTRIPLE t = *( pRgb++ );
            res[p] = t;
        }
    }
    DeleteObject( hBmp );
    return S_OK;
}

HRESULT LoadBitmapResource( CArray2D<RGBTRIPLE> &res, UINT iResID )
{
    const HMODULE hModule = NULL;
    HRSRC rc = FindResource( hModule, MAKEINTRESOURCE(iResID), RT_BITMAP );
    if(!rc) return GetLastHr();
    HGLOBAL hg = LoadResource( hModule, rc );
    if(!hg) return GetLastHr();
    const BITMAPINFO* pBmi = (const BITMAPINFO*)LockResource(hg);

    if( pBmi->bmiHeader.biPlanes != 1 || 
        pBmi->bmiHeader.biBitCount != 24 )
    {
        MyTrace( "The image must be 24 bit\n" );
        return HRESULT_FROM_WIN32( ERROR_INVALID_PIXEL_FORMAT );
    }

    const CSize sz( pBmi->bmiHeader.biWidth, pBmi->bmiHeader.biHeight );
    if( sz.cx < 3 || sz.cy < 3 )
    {
        MyTrace( "The image size must be at least 2x2 pixels\n" );
        return HRESULT_FROM_WIN32( ERROR_INCORRECT_SIZE );
    }
    if(!res.SetSize(sz))
        return E_OUTOFMEMORY;

    const int iBytesPerLine =  ( (sz.cx * pBmi->bmiHeader.biBitCount + 31) & (~31) ) / 8;

    const BYTE* pData = (BYTE*)(&pBmi->bmiColors[0]);
    CPoint p;
    for( p.y=0; p.y<sz.cy; p.y++ )
    {
        const size_t szOffset = iBytesPerLine * ( sz.cy - p.y - 1 );
        const RGBTRIPLE* pRgb = (const RGBTRIPLE*)( pData + szOffset );
        for( p.x=0; p.x<sz.cx; p.x++ )
        {
            const RGBTRIPLE t = *( pRgb++ );
            res[p] = t;
        }
    }
    return S_OK;
}