#pragma once

// This abstract class implements a mouse processing
// The derived class must implement HitTest function,
// to divide the window's client area into several zones.
// then the derived class will receive various mouse messages for each zone.
//
// The derived class doesn't need to override each event handlers, only the handlers he's interested in.
// The default event handlers do nothing
class CMouseProcessor
{
public:
	CMouseProcessor();
	~CMouseProcessor();
protected:
	// Do your own hit test
    // The pt is a mouse coordinates relative to the window's client area
    virtual int HitTest( CPoint pt )=0;

	// Is called when the mouse enters or leaves the zone
    virtual void OnMouseInOut( int iCtrl, bool bIn );

    // Is called when the mouse stays in a specific zone for SPI_GETMOUSEHOVERTIME time.
	virtual void OnMouseHover( int iCtrl, CPoint pt );

    // Below is the mouse buttons events.
    // vk argument is either VK_LBUTTON or VK_RBUTTON

    // Is called when the user presses mouse button in a zone,
    // or moves the mouse inside the zone with a button hold down if he first clicked the zone
	virtual void OnMouseDown( int iCtrl, DWORD vk );

    // Is called when the user releases mouse button in a zone,
    // or moves the mouse outside the zone with a button hold down
	virtual void OnMouseUp( int iCtrl, DWORD vk );

    // Is called when a used clicks the zone,
    // i.e. when he subsequently presses and releases the button in a single zone.
	virtual void OnMouseClick( int iCtrl, DWORD vk );

	static const int idTimer = 101;
private:
	BEGIN_MSG_MAP(CMouseProcessor)
		m_WindowHandle = hWnd;
		MESSAGE_HANDLER(WM_TIMER, OnWmTimer)
		MESSAGE_HANDLER(WM_MOUSEMOVE, OnWmMouseMove)
		MESSAGE_HANDLER(WM_MOUSELEAVE, OnWmMouseLeave)

		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnWmMouseButton)
		MESSAGE_HANDLER(WM_LBUTTONUP, OnWmMouseButton)
		MESSAGE_HANDLER(WM_RBUTTONDOWN, OnWmMouseButton)
		MESSAGE_HANDLER(WM_RBUTTONUP, OnWmMouseButton)
	END_MSG_MAP()

	// Mouse WM handlers
	LRESULT OnWmMouseMove(UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled);
	LRESULT OnWmMouseLeave( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled );
	LRESULT OnWmTimer(UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled);
	LRESULT OnWmMouseButton(UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled);

	void DoButtonUpDown( int nID, bool bDown );

	// SPI_GETMOUSEHOVERTIME
	UINT m_MouseHoverTime;

	int m_PrevHitTest;

	int m_idLeftBtn, m_idRightBtn;
	bool m_bLeftBtn, m_bRightBtn;

	DWORD m_MouseInTime;
	UINT_PTR m_Timer;
	bool m_bMouseTracking;
	HWND m_WindowHandle;

	bool m_bFirst;
};