#include "stdafx.h"
#include "ViewWnd.h"
#include "../resource.h"

const UINT CViewWnd::s_MapsIDs[] = { IDB_MAP1, IDB_MAP2 };

CViewWnd::CViewWnd( CWindow* pStatus ):
    m_pStatus(pStatus)
{
    m_dcMem = NULL;
    m_bmpPrev = NULL;
    m_cellTracking = -1;
    m_currMapID = 0;
}

CViewWnd::~CViewWnd()
{
}

HRESULT CViewWnd::Create( CWindow& parent )
{
//    HRESULT hr = LoadBmp(L"c:\\Dev\\PathfindingDemo\\_data\\05.bmp");
    HRESULT hr = LoadBmp( IDB_MAP1 );
    if(FAILED(hr))
        return hr;
    HWND res = __super::Create( parent.m_hWnd, NULL, NULL, WS_CHILD | WS_VISIBLE );
    if(!res)
        return GetLastHr();
    return S_OK;
}

HRESULT CViewWnd::LoadBmp( LPCTSTR strBmp )
{
    HRESULT hr = m_map.LoadBmp( strBmp, false );
    if(FAILED(hr))
        return hr;
    hr = __super::Initialize(m_map.GetSize());
    if(FAILED(hr))
        return hr;
    return S_OK;
}

HRESULT CViewWnd::LoadBmp( UINT resID )
{
    HRESULT hr = m_map.LoadBmp( resID );
    if(FAILED(hr))
        return hr;
    hr = __super::Initialize(m_map.GetSize());
    if(FAILED(hr))
        return hr;
    return S_OK;
}

void CViewWnd::PaintWithColor( HDC dc, const CRect& rcToPaint, COLORREF c )
{
    HBRUSH br = ::CreateSolidBrush(c);
    ::FillRect( dc, &rcToPaint, br );
    ::DeleteObject( br );
}

void CViewWnd::PaintCell( HDC dc, const CPoint& ptCell, const CRect& rcToPaint )
{
    size_t ind = m_map.GetIndex( ptCell );
    const CPathMapWin::CELL_INFO ci = m_map.getCellInfo(ptCell);

    BYTE bBlue = ci.isOnTheWave ? 128 : 0;

    if( ind == m_cellTracking )
    {
        PaintWithColor( dc,  rcToPaint, RGB(255,255,bBlue));
    }
    else
    {
        if( m_map.isTurretBody(ptCell, NULL ) )
        {
            PaintWithColor( dc,  rcToPaint, RGB(128,0,bBlue));
            return;
        }

        if( !ci.isPassable )
        {
            HBRUSH br = (HBRUSH)(GetStockObject(BLACK_BRUSH));
            ::FillRect( dc, &rcToPaint, br );
            return;
        }

        if( ci.isDestination )
        {
            PaintWithColor( dc,  rcToPaint, RGB(0,255,bBlue));
            return;
        }

        const BYTE bDanger = ( ci.cost > 255 ) ? 255: BYTE( ci.cost );
        if( !bBlue )
            bBlue = 255-bDanger;
        COLORREF c = RGB( 255, 255-bDanger, bBlue  );
        PaintWithColor( dc,  rcToPaint, c );
    }

    DrawArrow( dc, rcToPaint, ci.directionForward );
}

void CViewWnd::DrawArrow( HDC dc, const CRect& rcToPaint, BYTE dir )
{
    if( !m_dcMem )
    {
        m_dcMem = ::CreateCompatibleDC(dc);
        
        HBITMAP bmpArrows = LoadBitmap(_AtlBaseModule.GetModuleInstance(),
            MAKEINTRESOURCE( IDB_ARROWS ) );
        m_bmpPrev = (HBITMAP)( ::SelectObject( m_dcMem, bmpArrows ) );
    }
    
    BitBlt( dc,
        rcToPaint.left, rcToPaint.top, rcToPaint.Width(), rcToPaint.Height(),
        m_dcMem, 0, int(dir) * 12, SRCAND );
}

LRESULT CViewWnd::OnWmTimer( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    if( wParam != idTimer )
    {
        bHandled = FALSE;
        return 0;
    }
    static int USN = 0;
    USN++;
    if( 0 == USN % iUpdateFreq )
    {
        CRect rcUpdated;
        m_map.RecalcIncremental( iPixelsToUpdate, rcUpdated );
        if(!rcUpdated.IsRectEmpty())
            InvalidateCells(rcUpdated);
    }

    if( m_cellTracking < 0 )
        return 0;

    const CPoint cOld = m_map.GetIndex( m_cellTracking );
    const CPathMapWin::CELL_INFO ciOld = m_map.getCellInfo( cOld );
    if( ciOld.isDestination )
    {
        RepaintCell(cOld);
        return 0;
    }

    const CPoint cNew = cOld + m_map.s_Dirs[ciOld.directionForward];

    m_cellTracking = m_map.GetIndex(cNew);
    RepaintCell(cOld);
    RepaintCell(cNew);
    return 0;
}

void CViewWnd::OnMouseClick( int iCtrl, DWORD vk )
{
    // Test 
    if( vk == VK_LBUTTON )
    {
        if( m_cellTracking >= 0 )
        {
            CPoint p = m_map.GetIndex( m_cellTracking );
            m_cellTracking = -1;
            RepaintCell( p );
        }
        m_cellTracking = iCtrl;
        if( iCtrl < 0 )
            return;
        const CPoint c = m_map.GetIndex( iCtrl );
        const CPathMapWin::CELL_INFO ci = m_map.getCellInfo(c);
        if( !ci.isPassable || ci.isDestination )
            return;
        RepaintCell(c);
        return;
    }
    // Turret op
    if( vk == VK_RBUTTON )
    {
        const CPoint c = m_map.GetIndex( iCtrl );

        if( GetKeyState(VK_CONTROL) & 0x8000 )
        {
            size_t indTurret;
            if( !m_map.isTurretBody( c, &indTurret ) )
                return;
            m_map.RemoveTurret( indTurret );
            RedrawWindow();
            return;
        }

        CPathMapWin::TurretInfo ti;
        ti.Position = c;
        ti.Damage = 32;
        ti.Radius = 6;
        m_map.AddTurret( ti );
        RedrawWindow();
    }
}

LRESULT CViewWnd::OnWmKeyDown( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    if( wParam == VK_RETURN )
    {
        m_currMapID = ( m_currMapID + 1 ) % ( sizeof(s_MapsIDs)/sizeof(s_MapsIDs[0]) );
        m_cellTracking = -1;
        m_map.CleanupTurrets();
        LoadBmp(s_MapsIDs[m_currMapID]);
        RecalcLayout();
        RedrawWindow();
    }
    if( wParam == VK_F5 )
    {
        m_map.UpdateWholeGraph();
        RedrawWindow();
    }
    return 0;
}

LRESULT CViewWnd::OnWmCreate( UINT nMsg, WPARAM wParam,  LPARAM lParam, BOOL& bHandled )
{
    bHandled = FALSE;
    SetTimer( idTimer, dwTimerFrequency, NULL );

    return 0;
}

void CViewWnd::OnMouseInOut( int iCtrl, bool bIn )
{
    if( !bIn )
        return;
    CString str = m_map.GetCellInfo( m_map.GetIndex( iCtrl ) );
    m_pStatus->SetWindowText( str );
}